/*
*  Power BI Visualizations
*
*  Copyright (c) Microsoft Corporation
*  All rights reserved.
*  MIT License
*
*  Permission is hereby granted, free of charge, to any person obtaining a copy
*  of this software and associated documentation files (the ""Software""), to deal
*  in the Software without restriction, including without limitation the rights
*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
*  copies of the Software, and to permit persons to whom the Software is
*  furnished to do so, subject to the following conditions:
*
*  The above copyright notice and this permission notice shall be included in
*  all copies or substantial portions of the Software.
*
*  THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
*  THE SOFTWARE.
*/
var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var powerbi;
(function (powerbi) {
    var visuals;
    (function (visuals) {
        var Timeline1447991079100;
        (function (Timeline1447991079100) {
            var createClassAndSelector = jsCommon.CssConstants.createClassAndSelector;
            var SelectionManager = visuals.utility.SelectionManager;
            var px = jsCommon.PixelConverter.toString;
            var pt = jsCommon.PixelConverter.fromPoint;
            var fromPointToPixel = jsCommon.PixelConverter.fromPointToPixel;
            Timeline1447991079100.Months = powerbi.createEnumType([
                { value: 1, displayName: 'January' },
                { value: 2, displayName: 'February' },
                { value: 3, displayName: 'March' },
                { value: 4, displayName: 'April' },
                { value: 5, displayName: 'May' },
                { value: 6, displayName: 'June' },
                { value: 7, displayName: 'July' },
                { value: 8, displayName: 'August' },
                { value: 9, displayName: 'September' },
                { value: 10, displayName: 'October' },
                { value: 11, displayName: 'November' },
                { value: 12, displayName: 'December' }
            ]);
            Timeline1447991079100.WeekDays = powerbi.createEnumType([
                { value: 0, displayName: 'Sunday' },
                { value: 1, displayName: 'Monday' },
                { value: 2, displayName: 'Tuesday' },
                { value: 3, displayName: 'Wednesday' },
                { value: 4, displayName: 'Thursday' },
                { value: 5, displayName: 'Friday' },
                { value: 6, displayName: 'Saturday' }
            ]);
            (function (GranularityType) {
                GranularityType[GranularityType["year"] = 0] = "year";
                GranularityType[GranularityType["quarter"] = 1] = "quarter";
                GranularityType[GranularityType["month"] = 2] = "month";
                GranularityType[GranularityType["week"] = 3] = "week";
                GranularityType[GranularityType["day"] = 4] = "day";
            })(Timeline1447991079100.GranularityType || (Timeline1447991079100.GranularityType = {}));
            var GranularityType = Timeline1447991079100.GranularityType;
            var SelectedCellColorProp = { objectName: 'cells', propertyName: 'fillSelected' };
            var UnselectedCellColorProp = { objectName: 'cells', propertyName: 'fillUnselected' };
            var ScaleColorProp = { objectName: 'granularity', propertyName: 'scaleColor' };
            var SliderColorProp = { objectName: 'granularity', propertyName: 'sliderColor' };
            var TimeRangeColorProp = { objectName: 'rangeHeader', propertyName: 'fontColor' };
            var TimeRangeSizeProp = { objectName: 'rangeHeader', propertyName: 'textSize' };
            var TimeRangeShowProp = { objectName: 'rangeHeader', propertyName: 'show' };
            var LabelsColorProp = { objectName: 'labels', propertyName: 'fontColor' };
            var LabelsSizeProp = { objectName: 'labels', propertyName: 'textSize' };
            var LabelsShowProp = { objectName: 'labels', propertyName: 'show' };
            var CalendarMonthProp = { objectName: 'calendar', propertyName: 'month' };
            var CalendarDayProp = { objectName: 'calendar', propertyName: 'day' };
            var WeekDayProp = { objectName: 'weekDay', propertyName: 'day' };
            var GranularityNames = [
                {
                    granularityType: 0 /* year */,
                    name: "year"
                },
                {
                    granularityType: 1 /* quarter */,
                    name: "quarter"
                },
                {
                    granularityType: 2 /* month */,
                    name: "month"
                },
                {
                    granularityType: 3 /* week */,
                    name: "week"
                },
                {
                    granularityType: 4 /* day */,
                    name: "day"
                }
            ];
            var TimelineGranularity = (function () {
                function TimelineGranularity() {
                    this.datePeriods = [];
                }
                /**
                * Returns the short month name of the given date (e.g. Jan, Feb, Mar)
                */
                TimelineGranularity.prototype.shortMonthName = function (date) {
                    return date.toString().split(' ')[1];
                };
                TimelineGranularity.prototype.resetDatePeriods = function () {
                    this.datePeriods = [];
                };
                TimelineGranularity.prototype.getDatePeriods = function () {
                    return this.datePeriods;
                };
                TimelineGranularity.prototype.getExtendedLabel = function () {
                    return this.extendedLabel;
                };
                TimelineGranularity.prototype.setExtendedLabel = function (extendedLabel) {
                    this.extendedLabel = extendedLabel;
                };
                TimelineGranularity.prototype.createLabels = function (granularity) {
                    var labels = [];
                    var lastDatePeriod;
                    _.map(this.datePeriods, function (x) {
                        if (_.isEmpty(labels) || !granularity.sameLabel(x, lastDatePeriod)) {
                            lastDatePeriod = x;
                            labels.push(granularity.generateLabel(x));
                        }
                    });
                    return labels;
                };
                /**
                * Adds the new date into the given datePeriods array
                * If the date corresponds to the last date period, given the current granularity,
                * it will be added to that date period. Otherwise, a new date period will be added to the array.
                * i.e. using Month granularity, Feb 2 2015 corresponds to Feb 3 2015.
                * It is assumed that the given date does not correspond to previous date periods, other than the last date period
                */
                TimelineGranularity.prototype.addDate = function (date, identifierArray) {
                    var datePeriods = this.getDatePeriods();
                    var lastDatePeriod = datePeriods[datePeriods.length - 1];
                    if (datePeriods.length === 0 || !_.isEqual(lastDatePeriod.identifierArray, identifierArray)) {
                        if (datePeriods.length > 0)
                            lastDatePeriod.endDate = date;
                        datePeriods.push({
                            identifierArray: identifierArray,
                            startDate: date,
                            endDate: date,
                            week: this.determineWeek(date),
                            year: this.determineYear(date),
                            fraction: 1,
                            index: datePeriods.length
                        });
                    }
                    else
                        lastDatePeriod.endDate = date;
                };
                TimelineGranularity.prototype.setNewEndDate = function (date) {
                    _.last(this.datePeriods).endDate = date;
                };
                /**
                 * Splits a given period into two periods.
                 * The new period is added after the index of the old one, while the old one is simply updated.
                 * @param index The index of the date priod to be split
                 * @param newFraction The fraction value of the new date period
                 * @param newDate The date in which the date period is split
                 */
                TimelineGranularity.prototype.splitPeriod = function (index, newFraction, newDate) {
                    var oldDatePeriod = this.datePeriods[index];
                    oldDatePeriod.fraction -= newFraction;
                    var newDateObject = {
                        identifierArray: oldDatePeriod.identifierArray,
                        startDate: newDate,
                        endDate: oldDatePeriod.endDate,
                        week: this.determineWeek(newDate),
                        year: this.determineYear(newDate),
                        fraction: newFraction,
                        index: oldDatePeriod.index + oldDatePeriod.fraction
                    };
                    oldDatePeriod.endDate = newDate;
                    this.datePeriods.splice(index + 1, 0, newDateObject);
                };
                TimelineGranularity.prototype.previousMonth = function (month) {
                    return (month > 0) ? month - 1 : 11;
                };
                TimelineGranularity.prototype.nextMonth = function (month) {
                    return (month < 11) ? month + 1 : 0;
                };
                TimelineGranularity.prototype.countWeeks = function (startDate, endDate) {
                    var totalDays;
                    if (endDate.getFullYear() === startDate.getFullYear() && endDate.getMonth() === startDate.getMonth() && endDate.getDate() >= startDate.getDate())
                        totalDays = endDate.getDate() - startDate.getDate();
                    else {
                        totalDays = endDate.getDate() - 1;
                        var lastMonth = this.nextMonth(startDate.getMonth());
                        var month = endDate.getMonth();
                        while (month !== lastMonth) {
                            totalDays += new Date(endDate.getFullYear(), month, 0).getDate();
                            month = this.previousMonth(month);
                        }
                        totalDays += new Date(endDate.getFullYear(), lastMonth, 0).getDate() - startDate.getDate();
                    }
                    return 1 + Math.floor(totalDays / 7);
                };
                TimelineGranularity.prototype.determineWeek = function (date) {
                    var year = date.getFullYear();
                    if (this.inPreviousYear(date))
                        year--;
                    var dateOfFirstWeek = Timeline.calendar.getDateOfFirstWeek(year);
                    var weeks = this.countWeeks(dateOfFirstWeek, date);
                    return [weeks, year];
                };
                TimelineGranularity.prototype.inPreviousYear = function (date) {
                    var dateOfFirstWeek = Timeline.calendar.getDateOfFirstWeek(date.getFullYear());
                    return date < dateOfFirstWeek;
                };
                TimelineGranularity.prototype.determineYear = function (date) {
                    var firstDay = new Date(date.getFullYear(), Timeline.calendar.getFirstMonthOfYear(), Timeline.calendar.getFirstDayOfYear());
                    return date.getFullYear() - ((firstDay <= date) ? 0 : 1);
                };
                return TimelineGranularity;
            })();
            Timeline1447991079100.TimelineGranularity = TimelineGranularity;
            var DayGranularity = (function (_super) {
                __extends(DayGranularity, _super);
                function DayGranularity() {
                    _super.apply(this, arguments);
                }
                DayGranularity.prototype.getType = function () {
                    return 4 /* day */;
                };
                DayGranularity.prototype.splitDate = function (date) {
                    return [this.shortMonthName(date), date.getDate(), date.getFullYear()];
                };
                DayGranularity.prototype.sameLabel = function (firstDatePeriod, secondDatePeriod) {
                    return firstDatePeriod.startDate.getTime() === secondDatePeriod.startDate.getTime();
                };
                DayGranularity.prototype.generateLabel = function (datePeriod) {
                    return {
                        title: this.shortMonthName(datePeriod.startDate) + ' ' + datePeriod.startDate.getDate() + ' - ' + datePeriod.year,
                        text: datePeriod.startDate.getDate().toString(),
                        id: datePeriod.index
                    };
                };
                return DayGranularity;
            })(TimelineGranularity);
            Timeline1447991079100.DayGranularity = DayGranularity;
            var MonthGranularity = (function (_super) {
                __extends(MonthGranularity, _super);
                function MonthGranularity() {
                    _super.apply(this, arguments);
                }
                MonthGranularity.prototype.getType = function () {
                    return 2 /* month */;
                };
                MonthGranularity.prototype.splitDate = function (date) {
                    return [this.shortMonthName(date), date.getFullYear()];
                };
                MonthGranularity.prototype.sameLabel = function (firstDatePeriod, secondDatePeriod) {
                    return this.shortMonthName(firstDatePeriod.startDate) === this.shortMonthName(secondDatePeriod.startDate);
                };
                MonthGranularity.prototype.generateLabel = function (datePeriod) {
                    var shortMonthName = this.shortMonthName(datePeriod.startDate);
                    return {
                        title: shortMonthName,
                        text: shortMonthName,
                        id: datePeriod.index
                    };
                };
                return MonthGranularity;
            })(TimelineGranularity);
            Timeline1447991079100.MonthGranularity = MonthGranularity;
            var WeekGranularity = (function (_super) {
                __extends(WeekGranularity, _super);
                function WeekGranularity() {
                    _super.apply(this, arguments);
                }
                WeekGranularity.prototype.getType = function () {
                    return 3 /* week */;
                };
                WeekGranularity.prototype.splitDate = function (date) {
                    return this.determineWeek(date);
                };
                WeekGranularity.prototype.sameLabel = function (firstDatePeriod, secondDatePeriod) {
                    return _.isEqual(firstDatePeriod.week, secondDatePeriod.week);
                };
                WeekGranularity.prototype.generateLabel = function (datePeriod) {
                    return {
                        title: 'Week ' + datePeriod.week[0] + ' - ' + datePeriod.week[1],
                        text: 'W' + datePeriod.week[0],
                        id: datePeriod.index
                    };
                };
                return WeekGranularity;
            })(TimelineGranularity);
            Timeline1447991079100.WeekGranularity = WeekGranularity;
            var QuarterGranularity = (function (_super) {
                __extends(QuarterGranularity, _super);
                function QuarterGranularity() {
                    _super.apply(this, arguments);
                }
                /**
                 * Returns the date's quarter name (e.g. Q1, Q2, Q3, Q4)
                 * @param date A date
                 */
                QuarterGranularity.prototype.quarterText = function (date) {
                    var quarter = 3;
                    var year = date.getFullYear();
                    while (date < Timeline.calendar.getQuarterStartDate(year, quarter))
                        if (quarter > 0)
                            quarter--;
                        else {
                            quarter = 3;
                            year--;
                        }
                    quarter++;
                    return 'Q' + quarter;
                };
                QuarterGranularity.prototype.getType = function () {
                    return 1 /* quarter */;
                };
                QuarterGranularity.prototype.splitDate = function (date) {
                    return [this.quarterText(date), date.getFullYear()];
                };
                QuarterGranularity.prototype.sameLabel = function (firstDatePeriod, secondDatePeriod) {
                    return this.quarterText(firstDatePeriod.startDate) === this.quarterText(secondDatePeriod.startDate) && firstDatePeriod.year === secondDatePeriod.year;
                };
                QuarterGranularity.prototype.generateLabel = function (datePeriod) {
                    var quarter = this.quarterText(datePeriod.startDate);
                    return {
                        title: quarter + ' ' + datePeriod.year,
                        text: quarter,
                        id: datePeriod.index
                    };
                };
                return QuarterGranularity;
            })(TimelineGranularity);
            Timeline1447991079100.QuarterGranularity = QuarterGranularity;
            var YearGranularity = (function (_super) {
                __extends(YearGranularity, _super);
                function YearGranularity() {
                    _super.apply(this, arguments);
                }
                YearGranularity.prototype.getType = function () {
                    return 0 /* year */;
                };
                YearGranularity.prototype.splitDate = function (date) {
                    return [date.getFullYear()];
                };
                YearGranularity.prototype.sameLabel = function (firstDatePeriod, secondDatePeriod) {
                    return firstDatePeriod.year === secondDatePeriod.year;
                };
                YearGranularity.prototype.generateLabel = function (datePeriod) {
                    return {
                        title: 'Year ' + datePeriod.year,
                        text: datePeriod.year.toString(),
                        id: datePeriod.index
                    };
                };
                return YearGranularity;
            })(TimelineGranularity);
            Timeline1447991079100.YearGranularity = YearGranularity;
            var TimelineGranularityData = (function () {
                function TimelineGranularityData(startDate, endDate) {
                    this.granularities = [];
                    this.setDatesRange(startDate, endDate);
                    var lastDate = this.dates[this.dates.length - 1];
                    this.endingDate = TimelineGranularityData.nextDay(lastDate);
                }
                /**
                 * Returns the date of the previos day
                 * @param date The following date
                 */
                TimelineGranularityData.previousDay = function (date) {
                    var prevDay = new Date(date.getTime());
                    prevDay.setDate(prevDay.getDate() - 1);
                    return prevDay;
                };
                /**
                 * Returns the date of the next day
                 * @param date The previous date
                 */
                TimelineGranularityData.nextDay = function (date) {
                    var nextDay = new Date(date.getTime());
                    nextDay.setDate(nextDay.getDate() + 1);
                    return nextDay;
                };
                /**
                * Returns an array of dates with all the days between the start date and the end date
                */
                TimelineGranularityData.prototype.setDatesRange = function (startDate, endDate) {
                    this.dates = [];
                    var date = startDate;
                    while (date <= endDate) {
                        this.dates.push(date);
                        date = TimelineGranularityData.nextDay(date);
                    }
                };
                /**
                 * Adds a new granularity to the array of granularities.
                 * Resets the new granularity, adds all dates to it, and then edits the last date period with the ending date.
                 * @param granularity The new granularity to be added
                 */
                TimelineGranularityData.prototype.addGranularity = function (granularity) {
                    granularity.resetDatePeriods();
                    for (var _i = 0, _a = this.dates; _i < _a.length; _i++) {
                        var date = _a[_i];
                        var identifierArray = granularity.splitDate(date);
                        granularity.addDate(date, identifierArray);
                    }
                    granularity.setNewEndDate(this.endingDate);
                    this.granularities.push(granularity);
                };
                /**
                 * Returns a specific granularity from the array of granularities
                 * @param index The index of the requested granularity
                 */
                TimelineGranularityData.prototype.getGranularity = function (index) {
                    return this.granularities[index];
                };
                TimelineGranularityData.prototype.createGranularities = function () {
                    this.granularities = [];
                    this.addGranularity(new YearGranularity());
                    this.addGranularity(new QuarterGranularity());
                    this.addGranularity(new MonthGranularity());
                    this.addGranularity(new WeekGranularity());
                    this.addGranularity(new DayGranularity());
                };
                TimelineGranularityData.prototype.createLabels = function () {
                    var _this = this;
                    this.granularities.forEach(function (x) {
                        x.setExtendedLabel({
                            dayLabels: x.getType() >= 4 /* day */ ? x.createLabels(_this.granularities[4 /* day */]) : [],
                            weekLabels: x.getType() >= 3 /* week */ ? x.createLabels(_this.granularities[3 /* week */]) : [],
                            monthLabels: x.getType() >= 2 /* month */ ? x.createLabels(_this.granularities[2 /* month */]) : [],
                            quarterLabels: x.getType() >= 1 /* quarter */ ? x.createLabels(_this.granularities[1 /* quarter */]) : [],
                            yearLabels: x.getType() >= 0 /* year */ ? x.createLabels(_this.granularities[0 /* year */]) : [],
                        });
                    });
                };
                return TimelineGranularityData;
            })();
            Timeline1447991079100.TimelineGranularityData = TimelineGranularityData;
            var Utils = (function () {
                function Utils() {
                }
                /**
                 * Returns the date of the start of the selection
                 * @param timelineData The TimelineData which contains all the date periods
                 */
                Utils.getStartSelectionDate = function (timelineData) {
                    return timelineData.currentGranularity.getDatePeriods()[timelineData.selectionStartIndex].startDate;
                };
                /**
                 * Returns the date of the end of the selection
                 * @param timelineData The TimelineData which contains all the date periods
                 */
                Utils.getEndSelectionDate = function (timelineData) {
                    return timelineData.currentGranularity.getDatePeriods()[timelineData.selectionEndIndex].endDate;
                };
                /**
                 * Returns the date period of the end of the selection
                 * @param timelineData The TimelineData which contains all the date periods
                 */
                Utils.getEndSelectionPeriod = function (timelineData) {
                    return timelineData.currentGranularity.getDatePeriods()[timelineData.selectionEndIndex];
                };
                /**
                 * Returns the color of a cell, depending on whether its date period is between the selected date periods
                 * @param d The TimelineDataPoint of the cell
                 * @param timelineData The TimelineData with the selected date periods
                 * @param timelineFormat The TimelineFormat with the chosen colors
                 */
                Utils.getCellColor = function (d, timelineData, cellFormat) {
                    var inSelectedPeriods = d.datePeriod.startDate >= Utils.getStartSelectionDate(timelineData) && d.datePeriod.endDate <= Utils.getEndSelectionDate(timelineData);
                    return inSelectedPeriods ? cellFormat.colorInProperty : (cellFormat.colorOutProperty || 'transparent');
                };
                /**
                 * Returns the granularity type of the given granularity name
                 * @param granularityName The name of the granularity
                 */
                Utils.getGranularityType = function (granularityName) {
                    var index = _.findIndex(GranularityNames, function (x) { return x.name === granularityName; });
                    return GranularityNames[index].granularityType;
                };
                /**
                 * Returns the name of the granularity type
                 * @param granularity The type of granularity
                 */
                Utils.getGranularityName = function (granularity) {
                    var index = _.findIndex(GranularityNames, function (x) { return x.granularityType === granularity; });
                    return GranularityNames[index].name;
                };
                /**
                 * Splits the date periods of the current granularity, in case the stard and end of the selection is in between a date period.
                 * i.e. for a quarter granularity and a selection between Feb 6 and Dec 23, the date periods for Q1 and Q4 will be split accordingly
                 * @param timelineData The TimelineData that contains the date periods
                 * @param startDate The starting date of the selection
                 * @param endDate The ending date of the selection
                 */
                Utils.separateSelection = function (timelineData, startDate, endDate) {
                    var datePeriods = timelineData.currentGranularity.getDatePeriods();
                    var startDateIndex = _.findIndex(datePeriods, function (x) { return startDate < x.endDate; });
                    var endDateIndex = _.findIndex(datePeriods, function (x) { return endDate <= x.endDate; });
                    timelineData.selectionStartIndex = startDateIndex;
                    timelineData.selectionEndIndex = endDateIndex;
                    var startRatio = Utils.getDateRatio(datePeriods[startDateIndex], startDate, true);
                    var endRatio = Utils.getDateRatio(datePeriods[endDateIndex], endDate, false);
                    if (endRatio > 0)
                        timelineData.currentGranularity.splitPeriod(endDateIndex, endRatio, endDate);
                    if (startRatio > 0) {
                        var startFration = datePeriods[startDateIndex].fraction - startRatio;
                        timelineData.currentGranularity.splitPeriod(startDateIndex, startFration, startDate);
                        timelineData.selectionStartIndex++;
                        timelineData.selectionEndIndex++;
                    }
                };
                /**
                 * Returns the ratio of the given date compared to the whole date period.
                 * The ratio is calculated either from the start or the end of the date period.
                 * i.e. the ratio of Feb 7 2016 compared to the month of Feb 2016,
                 * is 0.2142 from the start of the month, or 0.7857 from the end of the month.
                 * @param datePeriod The date period that contain the specified date
                 * @param date The date
                 * @param fromStart Whether to calculater the ratio from the start of the date period.
                 */
                Utils.getDateRatio = function (datePeriod, date, fromStart) {
                    var dateDifference = fromStart ? date.getTime() - datePeriod.startDate.getTime() : datePeriod.endDate.getTime() - date.getTime();
                    var periodDifference = datePeriod.endDate.getTime() - datePeriod.startDate.getTime();
                    return periodDifference === 0 ? 0 : dateDifference / periodDifference;
                };
                /**
                * Returns the time range text, depending on the given granularity (e.g. "Feb 3 2014 - Apr 5 2015", "Q1 2014 - Q2 2015")
                */
                Utils.timeRangeText = function (timelineData) {
                    var startSelectionDateArray = timelineData.currentGranularity.splitDate(Utils.getStartSelectionDate(timelineData));
                    var endSelectionDateArray = timelineData.currentGranularity.splitDate(Utils.getEndSelectionPeriod(timelineData).startDate);
                    return startSelectionDateArray.join(' ') + ' - ' + endSelectionDateArray.join(' ');
                };
                Utils.dateRangeText = function (datePeriod) {
                    return datePeriod.startDate.toDateString() + ' - ' + TimelineGranularityData.previousDay(datePeriod.endDate).toDateString();
                };
                /**
                 * Combines the first two partial date periods, into a single date period.
                 * Returns whether a partial date period was found.
                 * i.e. combines "Feb 1 2016 - Feb 5 2016" with "Feb 5 2016 - Feb 29 2016" into "Feb 1 2016 - Feb 29 2016"
                 * @param datePeriods The list of date periods
                 */
                Utils.unseparateSelection = function (datePeriods) {
                    var separationIndex = _.findIndex(datePeriods, function (x) { return x.fraction < 1; });
                    if (separationIndex >= 0) {
                        datePeriods[separationIndex].endDate = datePeriods[separationIndex + 1].endDate;
                        datePeriods[separationIndex].fraction += datePeriods[separationIndex + 1].fraction;
                        datePeriods.splice(separationIndex + 1, 1);
                        return true;
                    }
                    return false;
                };
                return Utils;
            })();
            Timeline1447991079100.Utils = Utils;
            var Calendar = (function () {
                function Calendar(calendarFormat) {
                    var _this = this;
                    this.firstDayOfWeek = calendarFormat.weekDayProperty;
                    this.firstMonthOfYear = calendarFormat.firstMonthProperty - 1;
                    this.firstDayOfYear = calendarFormat.firstDayProperty;
                    this.dateOfFirstWeek = {};
                    this.quarterFirstMonths = [0, 3, 6, 9].map(function (x) { return x + _this.firstMonthOfYear; });
                }
                Calendar.prototype.getFirstDayOfWeek = function () {
                    return this.firstDayOfWeek;
                };
                Calendar.prototype.getFirstMonthOfYear = function () {
                    return this.firstMonthOfYear;
                };
                Calendar.prototype.getFirstDayOfYear = function () {
                    return this.firstDayOfYear;
                };
                Calendar.prototype.getQuarterStartDate = function (year, quarterIndex) {
                    return new Date(year, this.quarterFirstMonths[quarterIndex], this.firstDayOfYear);
                };
                Calendar.prototype.isChanged = function (calendarFormat) {
                    return this.firstMonthOfYear !== (calendarFormat.firstMonthProperty - 1) || this.firstDayOfYear !== calendarFormat.firstDayProperty || this.firstDayOfWeek !== calendarFormat.weekDayProperty;
                };
                Calendar.prototype.calculateDateOfFirstWeek = function (year) {
                    var date = new Date(year, this.firstMonthOfYear, this.firstDayOfYear);
                    while (date.getDay() !== this.firstDayOfWeek)
                        date = TimelineGranularityData.nextDay(date);
                    return date;
                };
                Calendar.prototype.getDateOfFirstWeek = function (year) {
                    if (!this.dateOfFirstWeek[year])
                        this.dateOfFirstWeek[year] = this.calculateDateOfFirstWeek(year);
                    return this.dateOfFirstWeek[year];
                };
                return Calendar;
            })();
            Timeline1447991079100.Calendar = Calendar;
            var Timeline = (function () {
                function Timeline() {
                    var _this = this;
                    this.requiresNoUpdate = false;
                    this.datasetsChangedState = false;
                    this.selector = ['Y', 'Q', 'M', 'W', 'D'];
                    this.timelineMargins = {
                        LeftMargin: 15,
                        RightMargin: 15,
                        TopMargin: 15,
                        BottomMargin: 10,
                        CellWidth: 40,
                        CellHeight: 25,
                        StartXpoint: 10,
                        StartYpoint: 20,
                        ElementWidth: 30,
                        MinCellWidth: 30,
                        MaxCellHeight: 60,
                        PeriodSlicerRectWidth: 15,
                        PeriodSlicerRectHeight: 23
                    };
                    this.defaultTimelineProperties = {
                        DefaultLabelsShow: true,
                        TimelineDefaultTextSize: 9,
                        TimelineDefaultCellColor: "#ADD8E6",
                        TimelineDefaultCellColorOut: "",
                        TimelineDefaultTimeRangeShow: true,
                        DefaultTimeRangeColor: "#777777",
                        DefaultLabelColor: "#777777",
                        DefaultScaleColor: "#000000",
                        DefaultSliderColor: "#AAAAAA",
                        DefaultGranularity: 2 /* month */,
                        DefaultFirstMonth: 1,
                        DefaultFirstDay: 1,
                        DefaultFirstWeekDay: 0
                    };
                    this.timelineSelectors = {
                        TimelineVisual: createClassAndSelector('Timeline'),
                        SelectionRangeContainer: createClassAndSelector('selectionRangeContainer'),
                        textLabel: createClassAndSelector('label'),
                        LowerTextCell: createClassAndSelector('lowerTextCell'),
                        UpperTextCell: createClassAndSelector('upperTextCell'),
                        UpperTextArea: createClassAndSelector('upperTextArea'),
                        LowerTextArea: createClassAndSelector('lowerTextArea'),
                        RangeTextArea: createClassAndSelector('rangeTextArea'),
                        CellsArea: createClassAndSelector('cellsArea'),
                        CursorsArea: createClassAndSelector('cursorsArea'),
                        MainArea: createClassAndSelector('mainArea'),
                        SelectionCursor: createClassAndSelector('selectionCursor'),
                        Cell: createClassAndSelector('cell'),
                        CellRect: createClassAndSelector('cellRect'),
                        VertLine: createClassAndSelector('timelineVertLine'),
                        TimelineSlicer: createClassAndSelector('timelineSlicer'),
                        PeriodSlicerGranularities: createClassAndSelector('periodSlicerGranularities'),
                        PeriodSlicerSelection: createClassAndSelector('periodSlicerSelection'),
                        PeriodSlicerSelectionRect: createClassAndSelector('periodSlicerSelectionRect'),
                        PeriodSlicerRect: createClassAndSelector('periodSlicerRect')
                    };
                    this.drag = d3.behavior.drag().origin(function (d) {
                        d.x = d.selectionIndex * _this.timelineProperties.cellWidth;
                        return d;
                    }).on("dragstart", function () {
                        _this.dragstarted();
                    }).on("drag", function (d) {
                        _this.dragged(d);
                    }).on("dragend", function () {
                        _this.dragended();
                    });
                }
                /**
                 * Changes the current granularity depending on the given granularity type
                 * Separates the new granularity's date periods which contain the start/end selection
                 * Unseparates the date periods of the previous granularity.
                 * @param granularity The new granularity type
                 */
                Timeline.prototype.changeGranularity = function (granularity, startDate, endDate) {
                    if (Utils.unseparateSelection(this.timelineData.currentGranularity.getDatePeriods()))
                        Utils.unseparateSelection(this.timelineData.currentGranularity.getDatePeriods());
                    this.timelineData.currentGranularity = this.timelineGranularityData.getGranularity(granularity);
                    Utils.separateSelection(this.timelineData, startDate, endDate);
                };
                Timeline.prototype.init = function (options) {
                    this.hostServices = options.host;
                    this.initialized = false;
                    var element = options.element;
                    this.selectionManager = new SelectionManager({ hostServices: options.host });
                    this.timelineProperties = {
                        element: element,
                        textYPosition: 50,
                        cellsYPosition: this.timelineMargins.TopMargin * 3 + 65,
                        topMargin: this.timelineMargins.TopMargin,
                        bottomMargin: this.timelineMargins.BottomMargin,
                        leftMargin: this.timelineMargins.LeftMargin,
                        startXpoint: this.timelineMargins.StartXpoint,
                        startYpoint: this.timelineMargins.StartYpoint,
                        cellWidth: this.timelineMargins.CellWidth,
                        cellHeight: this.timelineMargins.CellHeight,
                        elementWidth: this.timelineMargins.ElementWidth,
                        rightMargin: this.timelineMargins.RightMargin
                    };
                    this.body = d3.select(element.get(0));
                    this.timelineDiv = this.body.append('div');
                    this.svg = this.timelineDiv.append('svg').attr('width', px(options.viewport.width)).classed(this.timelineSelectors.TimelineVisual.class, true);
                    this.addWrappElements();
                };
                Timeline.prototype.addWrappElements = function () {
                    this.clearCatcher = visuals.appendClearCatcher(this.svg);
                    this.clearCatcher.data([this]).on("click", function (timeline) { return timeline.clear(); }).on("touchstart", function (timeline) { return timeline.clear(); });
                    this.rangeText = this.svg.append('g').classed(this.timelineSelectors.RangeTextArea.class, true).append('text');
                    this.mainGroupElement = this.svg.append('g').classed(this.timelineSelectors.MainArea.class, true);
                    this.yearLabelsElement = this.mainGroupElement.append('g');
                    this.quarterLabelsElement = this.mainGroupElement.append('g');
                    this.monthLabelsElement = this.mainGroupElement.append('g');
                    this.weekLabelsElement = this.mainGroupElement.append('g');
                    this.dayLabelsElement = this.mainGroupElement.append('g');
                    this.cellsElement = this.mainGroupElement.append('g').classed(this.timelineSelectors.CellsArea.class, true);
                    this.cursorGroupElement = this.svg.append('g').classed(this.timelineSelectors.CursorsArea.class, true);
                };
                Timeline.prototype.clear = function () {
                    if (this.initialized) {
                        this.selectionManager.clear();
                        if (this.timelineData) {
                            this.timelineData.selectionStartIndex = 0;
                            this.timelineData.selectionEndIndex = this.timelineData.currentGranularity.getDatePeriods().length - 1;
                            if (_.any(this.timelineData.timelineDatapoints, function (x) { return x.index % 1 !== 0; }))
                                this.selectPeriod(this.timelineData.currentGranularity.getType());
                            else {
                                Timeline.updateCursors(this.timelineData, this.timelineProperties.cellWidth);
                                this.fillCells(this.timelineFormat.cellFormat);
                                this.renderCursors(this.timelineData, this.timelineFormat, this.timelineProperties.cellHeight, this.timelineProperties.cellsYPosition);
                                this.renderTimeRangeText(this.timelineData, this.timelineFormat.rangeTextFormat);
                                this.fillColorGranularity(this.timelineFormat.granularityFormat);
                            }
                            this.setSelection(this.timelineData);
                        }
                    }
                };
                Timeline.prototype.drawGranular = function (timelineProperties) {
                    var _this = this;
                    var dragPeriodRectState = false;
                    var startXpoint = timelineProperties.startXpoint;
                    var startYpoint = timelineProperties.startYpoint;
                    var elementWidth = timelineProperties.elementWidth;
                    this.selectorContainer = this.svg.append('g').classed(this.timelineSelectors.TimelineSlicer.class, true);
                    this.selectorContainer.on('mouseleave', function (d) { return dragPeriodRectState = false; });
                    // create horiz. line
                    this.horizLine = this.selectorContainer.append('rect');
                    var selectorPeriods = this.selector;
                    this.horizLine.attr({
                        height: px(1),
                        x: px(startXpoint),
                        y: px(startYpoint + 2),
                        width: px((selectorPeriods.length - 1) * elementWidth),
                    });
                    // create vert. lines
                    this.vertLine = this.selectorContainer.selectAll("vertLines").data(selectorPeriods).enter().append('rect');
                    this.vertLine.classed(this.timelineSelectors.VertLine.class, true).attr({
                        x: function (d, index) { return px(startXpoint + index * elementWidth); },
                        y: px(startYpoint),
                        width: px(2),
                        height: px(3),
                    }).style({ 'cursor': 'pointer' });
                    // create text lables
                    var text = this.selectorContainer.selectAll(this.timelineSelectors.PeriodSlicerGranularities.selector).data(selectorPeriods).enter().append("text").classed(this.timelineSelectors.PeriodSlicerGranularities.class, true);
                    this.textLabels = text.text(function (d) { return d; }).attr({
                        x: function (d, index) { return px(startXpoint - 3 + index * elementWidth); },
                        y: px(startYpoint - 3),
                    });
                    // create selected period text
                    this.selectedText = this.selectorContainer.append("text").classed(this.timelineSelectors.PeriodSlicerSelection.class, true);
                    this.selectedText.text(Utils.getGranularityName(this.defaultTimelineProperties.DefaultGranularity)).attr({
                        x: px(startXpoint + 2 * elementWidth),
                        y: px(startYpoint + 17),
                    });
                    var selRects = this.selectorContainer.selectAll(this.timelineSelectors.PeriodSlicerSelectionRect.selector).data(selectorPeriods).enter().append('rect').classed(this.timelineSelectors.PeriodSlicerSelectionRect.class, true);
                    var clickHandler = function (d, index) {
                        _this.selectPeriod(index);
                        dragPeriodRectState = true;
                    };
                    selRects.attr({
                        x: function (d, index) { return px(startXpoint - elementWidth / 2 + index * elementWidth); },
                        y: px(3),
                        width: px(elementWidth),
                        height: px(23)
                    }).style({ 'cursor': 'pointer' }).on('mousedown', clickHandler).on('touchstart', clickHandler).on('mouseup', function () { return dragPeriodRectState = false; }).on('touchend', function () { return dragPeriodRectState = false; }).on("mouseover", function (d, index) {
                        if (dragPeriodRectState) {
                            _this.selectPeriod(index);
                        }
                    });
                    var dragPeriodRect = d3.behavior.drag().on("dragstart", function (e, b) {
                        dragPeriodRectState = true;
                    }).on("dragend", function (e, b) {
                        dragPeriodRectState = false;
                    });
                    this.periodSlicerRect = this.selectorContainer.append('rect').classed(this.timelineSelectors.PeriodSlicerRect.class, true).attr({
                        x: px(startXpoint - 6 + this.defaultTimelineProperties.DefaultGranularity * elementWidth),
                        y: px(startYpoint - 16),
                        rx: px(4),
                        width: px(15),
                        height: px(23),
                    }).on('mouseup', function (d) { return dragPeriodRectState = false; });
                    this.periodSlicerRect.call(dragPeriodRect);
                };
                Timeline.prototype.fillColorGranularity = function (granularityFormat) {
                    this.periodSlicerRect.style("stroke", granularityFormat.sliderColorProperty);
                    this.selectedText.attr('fill', granularityFormat.scaleColorProperty);
                    this.textLabels.attr('fill', granularityFormat.scaleColorProperty);
                    this.vertLine.attr('fill', granularityFormat.scaleColorProperty);
                    this.horizLine.attr('fill', granularityFormat.scaleColorProperty);
                };
                Timeline.prototype.redrawPeriod = function (granularity) {
                    var dx = this.timelineMargins.StartXpoint + granularity * this.timelineMargins.ElementWidth;
                    this.periodSlicerRect.transition().attr("x", px(dx - 7));
                    this.selectedText.text(Utils.getGranularityName(granularity));
                    var startDate = Utils.getStartSelectionDate(this.timelineData);
                    var endDate = Utils.getEndSelectionDate(this.timelineData);
                    this.changeGranularity(granularity, startDate, endDate);
                };
                Timeline.setMeasures = function (labelFormat, granularityType, datePeriodsCount, viewport, timelineProperties, timelineMargins) {
                    timelineProperties.cellsYPosition = timelineProperties.textYPosition;
                    var labelSize = fromPointToPixel(labelFormat.sizeProperty);
                    if (labelFormat.showProperty)
                        timelineProperties.cellsYPosition += labelSize * 1.5 * (granularityType + 1);
                    var svgHeight = Math.max(0, viewport.height - timelineMargins.TopMargin);
                    var maxHeight = viewport.width - timelineMargins.RightMargin - timelineMargins.MinCellWidth * datePeriodsCount;
                    var height = Math.max(timelineMargins.MinCellWidth, Math.min(timelineMargins.MaxCellHeight, maxHeight, svgHeight - timelineProperties.cellsYPosition - 20));
                    var width = Math.max(timelineMargins.MinCellWidth, (viewport.width - height - timelineMargins.RightMargin) / datePeriodsCount);
                    timelineProperties.cellHeight = height;
                    timelineProperties.cellWidth = width;
                };
                Timeline.prototype.visualChangeOnly = function (options) {
                    if (options && options.dataViews && options.dataViews[0] && options.dataViews[0].metadata && this.options && this.options.dataViews && this.options.dataViews[0] && this.options.dataViews[0].metadata) {
                        var newObjects = options.dataViews[0].metadata.objects;
                        var oldObjects = this.options.dataViews[0].metadata.objects;
                        var properties = ['rangeHeader', 'cells', 'labels', 'granularity'];
                        var metadataChanged = !properties.every(function (x) { return _.isEqual(newObjects ? newObjects[x] : undefined, oldObjects ? oldObjects[x] : undefined); });
                        return options.suppressAnimations || metadataChanged;
                    }
                    return false;
                };
                /**
                 * Note: Public for testability.
                 */
                Timeline.prototype.datasetsChanged = function (options) {
                    if (options && options.dataViews && options.dataViews[0] && options.dataViews[0].categorical && options.dataViews[0].categorical.categories && options.dataViews[0].categorical.categories[0] && options.dataViews[0].categorical.categories[0].source && this.options && this.options.dataViews && this.options.dataViews[0] && this.options.dataViews[0].categorical && this.options.dataViews[0].categorical.categories && this.options.dataViews[0].categorical.categories[0] && this.options.dataViews[0].categorical.categories[0].source) {
                        var newObjects = options.dataViews[0].categorical.categories[0].source.displayName;
                        var oldObjects = this.options.dataViews[0].categorical.categories[0].source.displayName;
                        if (!_.isEqual(newObjects, oldObjects))
                            return true;
                    }
                    return false;
                };
                Timeline.prototype.unavailableType = function (dataViewCategorical) {
                    return !dataViewCategorical.categories || dataViewCategorical.categories.length !== 1 || !dataViewCategorical.categories[0].values || dataViewCategorical.categories[0].values.length === 0 || !dataViewCategorical.categories[0].source || !dataViewCategorical.categories[0].source.type;
                };
                Timeline.prototype.unavailableChildIdentityField = function (dataViewTree) {
                    return !dataViewTree.root || !dataViewTree.root.childIdentityFields || dataViewTree.root.childIdentityFields.length === 0;
                };
                Timeline.prototype.createTimelineOptions = function (dataView) {
                    this.dataView = dataView;
                    if (!dataView.categorical || !dataView.metadata || this.unavailableType(dataView.categorical) || !dataView.tree || this.unavailableChildIdentityField(dataView.tree))
                        return false;
                    var columnExp = dataView.tree.root.childIdentityFields[0];
                    this.valueType = columnExp ? columnExp.ref : null;
                    if (!(dataView.categorical.categories[0].source.type.dateTime || (dataView.categorical.categories[0].source.type.numeric && (this.valueType === 'Year' || this.valueType === 'Date'))))
                        return false;
                    this.values = this.prepareValues(this.dataView.categorical.categories[0].values);
                    return true;
                };
                //Public for testability.
                Timeline.prototype.prepareValues = function (values) {
                    // remove null strings and rebuild string type date 
                    // (BUG #7266283 IN PBI-service)
                    values = values.filter(Boolean);
                    for (var i in values) {
                        var item = values[i];
                        if (typeof (item) === 'String' && (String(new Date(item)) !== 'Invalid Date')) {
                            return values[i] = new Date(item);
                        }
                    }
                    ;
                    return values;
                };
                Timeline.prototype.createTimelineData = function () {
                    var startDate;
                    var endDate;
                    if (this.valueType === 'Year') {
                        var years = this.values;
                        startDate = new Date(_.min(years), 0);
                        endDate = new Date(_.max(years), 11);
                    }
                    else {
                        var dates = this.values;
                        startDate = _.min(dates);
                        endDate = _.max(dates);
                    }
                    this.timelineFormat = Timeline.fillTimelineFormat(this.options.dataViews[0].metadata.objects, this.defaultTimelineProperties);
                    if (!this.initialized) {
                        this.drawGranular(this.timelineProperties);
                        this.fillColorGranularity(this.timelineFormat.granularityFormat);
                    }
                    if (this.initialized) {
                        var actualEndDate = TimelineGranularityData.nextDay(endDate);
                        var daysPeriods = this.timelineGranularityData.getGranularity(4 /* day */).getDatePeriods();
                        var prevStartDate = daysPeriods[0].startDate;
                        var prevEndDate = daysPeriods[daysPeriods.length - 1].endDate;
                        var changedSelection = startDate.getTime() >= prevStartDate.getTime() && actualEndDate.getTime() <= prevEndDate.getTime();
                        this.newGranularity = this.timelineData.currentGranularity.getType();
                        // this.fillColorGranularity(this.timelineFormat.granularityFormat);
                        if (changedSelection) {
                            this.changeGranularity(this.newGranularity, startDate, actualEndDate);
                        }
                        else {
                            if (actualEndDate < prevEndDate)
                                endDate = daysPeriods[daysPeriods.length - 1].startDate;
                            if (startDate > prevStartDate)
                                startDate = prevStartDate;
                            this.initialized = false;
                        }
                    }
                    if (!this.initialized) {
                        this.timelineGranularityData = new TimelineGranularityData(startDate, endDate);
                        this.timelineData = {
                            elementsCount: 0,
                            timelineDatapoints: [],
                            cursorDataPoints: new Array()
                        };
                    }
                };
                Timeline.prototype.update = function (options) {
                    var visualChange = this.visualChangeOnly(options);
                    this.datasetsChangedState = this.datasetsChanged(options);
                    if (this.timelineFormat) {
                        this.fillColorGranularity(this.timelineFormat.granularityFormat);
                    }
                    this.requiresNoUpdate = this.requiresNoUpdate && !this.datasetsChangedState && !visualChange;
                    if (this.requiresNoUpdate) {
                        this.requiresNoUpdate = false;
                        return;
                    }
                    this.options = options;
                    if (!options.dataViews || !options.dataViews[0])
                        return;
                    var validOptions = this.createTimelineOptions(options.dataViews[0]);
                    if (!validOptions) {
                        this.clearData();
                        return;
                    }
                    this.newGranularity = this.defaultTimelineProperties.DefaultGranularity;
                    if (!visualChange)
                        this.createTimelineData();
                    this.timelineFormat = Timeline.converter(this.timelineData, this.timelineProperties, this.defaultTimelineProperties, this.timelineGranularityData, options.dataViews[0], this.initialized, this.newGranularity, options.viewport, this.timelineMargins);
                    this.render(this.timelineData, this.timelineFormat, this.timelineProperties, options);
                    this.initialized = true;
                };
                Timeline.prototype.selectPeriod = function (periodNameIndex) {
                    this.redrawPeriod(periodNameIndex);
                    this.timelineFormat = Timeline.converter(this.timelineData, this.timelineProperties, this.defaultTimelineProperties, this.timelineGranularityData, this.options.dataViews[0], this.initialized, this.timelineData.currentGranularity.getType(), this.options.viewport, this.timelineMargins);
                    this.render(this.timelineData, this.timelineFormat, this.timelineProperties, this.options);
                };
                Timeline.isDataNotMatch = function (dataView) {
                    if (dataView.categorical.categories.length <= 0 || dataView.categorical.categories[0] === undefined || dataView.categorical.categories[0].identityFields === undefined || dataView.categorical.categories[0].identityFields.length <= 0)
                        return true;
                    return false;
                };
                Timeline.converter = function (timelineData, timelineProperties, defaultTimelineProperties, timelineGranularityData, dataView, initialized, granularityType, viewport, timelineMargins) {
                    var timelineFormat = Timeline.fillTimelineFormat(dataView.metadata.objects, defaultTimelineProperties);
                    if (!initialized) {
                        timelineData.cursorDataPoints.push({ x: 0, selectionIndex: 0, cursorIndex: 0 });
                        timelineData.cursorDataPoints.push({ x: 0, selectionIndex: 0, cursorIndex: 1 });
                    }
                    if (!initialized || Timeline.calendar.isChanged(timelineFormat.calendarFormat)) {
                        Timeline.calendar = new Calendar(timelineFormat.calendarFormat);
                        timelineGranularityData.createGranularities();
                        timelineGranularityData.createLabels();
                        timelineData.currentGranularity = timelineGranularityData.getGranularity(granularityType);
                        timelineData.selectionStartIndex = 0;
                        timelineData.selectionEndIndex = timelineData.currentGranularity.getDatePeriods().length - 1;
                    }
                    timelineData.categorySourceName = dataView.categorical.categories[0].source.displayName;
                    timelineData.columnIdentity = dataView.categorical.categories[0].identityFields[0];
                    if (dataView.categorical.categories[0].source.type.numeric) {
                        timelineData.columnIdentity.ref = "Date";
                    }
                    if (this.isDataNotMatch(dataView))
                        return;
                    var timelineElements = timelineData.currentGranularity.getDatePeriods();
                    timelineData.elementsCount = timelineElements.length;
                    timelineData.timelineDatapoints = [];
                    for (var _i = 0, timelineElements_1 = timelineElements; _i < timelineElements_1.length; _i++) {
                        var currentTimePeriod = timelineElements_1[_i];
                        var datapoint = {
                            index: currentTimePeriod.index,
                            datePeriod: currentTimePeriod
                        };
                        timelineData.timelineDatapoints.push(datapoint);
                    }
                    var countFullCells = timelineData.currentGranularity.getDatePeriods().filter(function (x) { return x.index % 1 === 0; }).length;
                    Timeline.setMeasures(timelineFormat.labelFormat, timelineData.currentGranularity.getType(), countFullCells, viewport, timelineProperties, timelineMargins);
                    Timeline.updateCursors(timelineData, timelineProperties.cellWidth);
                    return timelineFormat;
                };
                Timeline.prototype.render = function (timelineData, timelineFormat, timelineProperties, options) {
                    var timelineDatapointsCount = this.timelineData.timelineDatapoints.filter(function (x) { return x.index % 1 === 0; }).length;
                    this.svgWidth = 1 + this.timelineProperties.cellHeight + timelineProperties.cellWidth * timelineDatapointsCount;
                    this.renderTimeRangeText(timelineData, timelineFormat.rangeTextFormat);
                    this.fillColorGranularity(this.timelineFormat.granularityFormat);
                    this.timelineDiv.attr({
                        height: px(options.viewport.height),
                        width: px(options.viewport.width),
                        'drag-resize-disabled': true
                    }).style({
                        'overflow-x': 'auto',
                        'overflow-y': 'auto'
                    });
                    this.svg.attr({
                        height: px(Math.max(0, options.viewport.height - this.timelineMargins.TopMargin)),
                        width: px(Math.max(0, this.svgWidth))
                    });
                    var fixedTranslateString = visuals.SVGUtil.translate(timelineProperties.leftMargin, timelineProperties.topMargin);
                    var translateString = visuals.SVGUtil.translate(timelineProperties.cellHeight / 2, timelineProperties.topMargin);
                    this.mainGroupElement.attr('transform', translateString);
                    this.selectorContainer.attr('transform', fixedTranslateString);
                    this.cursorGroupElement.attr('transform', translateString);
                    var extendedLabels = this.timelineData.currentGranularity.getExtendedLabel();
                    var granularityType = this.timelineData.currentGranularity.getType();
                    var yPos = 0, yDiff = 1.50;
                    this.renderLabels(extendedLabels.yearLabels, this.yearLabelsElement, yPos, granularityType === 0);
                    yPos += yDiff;
                    this.renderLabels(extendedLabels.quarterLabels, this.quarterLabelsElement, yPos, granularityType === 1);
                    yPos += yDiff;
                    this.renderLabels(extendedLabels.monthLabels, this.monthLabelsElement, yPos, granularityType === 2);
                    yPos += yDiff;
                    this.renderLabels(extendedLabels.weekLabels, this.weekLabelsElement, yPos, granularityType === 3);
                    yPos += yDiff;
                    this.renderLabels(extendedLabels.dayLabels, this.dayLabelsElement, yPos, granularityType === 4);
                    this.renderCells(timelineData, timelineFormat, timelineProperties, options.suppressAnimations);
                    this.renderCursors(timelineData, timelineFormat, timelineProperties.cellHeight, timelineProperties.cellsYPosition);
                };
                Timeline.prototype.renderLabels = function (labels, labelsElement, index, isLast) {
                    var _this = this;
                    var labelTextSelection = labelsElement.selectAll(this.timelineSelectors.textLabel.selector);
                    if (!this.timelineFormat.labelFormat.showProperty) {
                        labelTextSelection.remove();
                        return;
                    }
                    var labelsGroupSelection = labelTextSelection.data(labels);
                    labelsGroupSelection.enter().append('text').classed(this.timelineSelectors.textLabel.class, true);
                    labelsGroupSelection.text(function (x, id) {
                        if (!isLast && id === 0 && labels.length > 1) {
                            var fontSize = pt(_this.timelineFormat.labelFormat.sizeProperty);
                            var textProperties = {
                                text: labels[0].text,
                                fontFamily: 'arial',
                                fontSize: fontSize
                            };
                            var halfFirstTextWidth = powerbi.TextMeasurementService.measureSvgTextWidth(textProperties) / 2;
                            textProperties = {
                                text: labels[1].text,
                                fontFamily: 'arial',
                                fontSize: fontSize
                            };
                            var halfSecondTextWidth = powerbi.TextMeasurementService.measureSvgTextWidth(textProperties) / 2;
                            var diff = _this.timelineProperties.cellWidth * (labels[1].id - labels[0].id);
                            if (diff < halfFirstTextWidth + halfSecondTextWidth)
                                return "";
                        }
                        var labelFormattedTextOptions = {
                            label: x.text,
                            maxWidth: _this.timelineProperties.cellWidth * (isLast ? 0.90 : 3),
                            fontSize: _this.timelineFormat.labelFormat.sizeProperty
                        };
                        return visuals.dataLabelUtils.getLabelFormattedText(labelFormattedTextOptions);
                    }).style('font-size', pt(this.timelineFormat.labelFormat.sizeProperty)).attr({
                        x: function (x) { return (x.id + 0.5) * _this.timelineProperties.cellWidth; },
                        y: this.timelineProperties.textYPosition + (1 + index) * fromPointToPixel(this.timelineFormat.labelFormat.sizeProperty),
                        fill: this.timelineFormat.labelFormat.colorProperty
                    }).append('title').text(function (x) { return x.title; });
                    labelsGroupSelection.exit().remove();
                };
                Timeline.prototype.clearData = function () {
                    this.initialized = false;
                    this.mainGroupElement.selectAll(this.timelineSelectors.CellRect.selector).remove();
                    this.mainGroupElement.selectAll(this.timelineSelectors.textLabel.selector).remove();
                    this.rangeText.text("");
                    this.cursorGroupElement.selectAll(this.timelineSelectors.SelectionCursor.selector).remove();
                    this.svg.select(this.timelineSelectors.TimelineSlicer.selector).remove();
                    this.mainGroupElement.selectAll(this.timelineSelectors.textLabel.selector);
                };
                Timeline.updateCursors = function (timelineData, cellWidth) {
                    var startDate = timelineData.timelineDatapoints[timelineData.selectionStartIndex].datePeriod;
                    timelineData.cursorDataPoints[0].selectionIndex = startDate.index;
                    var endDate = timelineData.timelineDatapoints[timelineData.selectionEndIndex].datePeriod;
                    timelineData.cursorDataPoints[1].selectionIndex = (endDate.index + endDate.fraction);
                };
                Timeline.fillTimelineFormat = function (objects, timelineProperties) {
                    var timelineFormat = {
                        rangeTextFormat: {
                            showProperty: powerbi.DataViewObjects.getValue(objects, TimeRangeShowProp, timelineProperties.TimelineDefaultTimeRangeShow),
                            colorProperty: powerbi.DataViewObjects.getFillColor(objects, TimeRangeColorProp, timelineProperties.DefaultTimeRangeColor),
                            sizeProperty: powerbi.DataViewObjects.getValue(objects, TimeRangeSizeProp, timelineProperties.TimelineDefaultTextSize)
                        },
                        cellFormat: {
                            colorInProperty: powerbi.DataViewObjects.getFillColor(objects, SelectedCellColorProp, timelineProperties.TimelineDefaultCellColor),
                            colorOutProperty: powerbi.DataViewObjects.getFillColor(objects, UnselectedCellColorProp, timelineProperties.TimelineDefaultCellColorOut)
                        },
                        granularityFormat: {
                            scaleColorProperty: powerbi.DataViewObjects.getFillColor(objects, ScaleColorProp, timelineProperties.DefaultScaleColor),
                            sliderColorProperty: powerbi.DataViewObjects.getFillColor(objects, SliderColorProp, timelineProperties.DefaultSliderColor)
                        },
                        labelFormat: {
                            showProperty: powerbi.DataViewObjects.getValue(objects, LabelsShowProp, timelineProperties.DefaultLabelsShow),
                            colorProperty: powerbi.DataViewObjects.getFillColor(objects, LabelsColorProp, timelineProperties.DefaultLabelColor),
                            sizeProperty: powerbi.DataViewObjects.getValue(objects, LabelsSizeProp, timelineProperties.TimelineDefaultTextSize)
                        },
                        calendarFormat: {
                            firstMonthProperty: powerbi.DataViewObjects.getValue(objects, CalendarMonthProp, 1),
                            firstDayProperty: Math.max(1, Math.min(31, powerbi.DataViewObjects.getValue(objects, CalendarDayProp, timelineProperties.DefaultFirstDay))),
                            weekDayProperty: Math.max(0, Math.min(6, powerbi.DataViewObjects.getValue(objects, WeekDayProp, timelineProperties.DefaultFirstWeekDay)))
                        }
                    };
                    return timelineFormat;
                };
                Timeline.prototype.fillCells = function (cellFormat) {
                    var _this = this;
                    var dataPoints = this.timelineData.timelineDatapoints;
                    var cellSelection = this.mainGroupElement.selectAll(this.timelineSelectors.CellRect.selector).data(dataPoints);
                    cellSelection.attr('fill', function (d) { return Utils.getCellColor(d, _this.timelineData, cellFormat); });
                };
                Timeline.prototype.renderCells = function (timelineData, timelineFormat, timelineProperties, suppressAnimations) {
                    var _this = this;
                    var allDataPoints = timelineData.timelineDatapoints;
                    var totalX = 0;
                    var cellsSelection = this.cellsElement.selectAll(this.timelineSelectors.CellRect.selector).data(allDataPoints);
                    cellsSelection.enter().append('rect').classed(this.timelineSelectors.CellRect.class, true);
                    cellsSelection.attr({
                        height: px(timelineProperties.cellHeight),
                        width: function (d) { return px(d.datePeriod.fraction * timelineProperties.cellWidth); },
                        x: function (d) {
                            var value = totalX;
                            totalX += d.datePeriod.fraction * timelineProperties.cellWidth;
                            return px(value);
                        },
                        y: px(timelineProperties.cellsYPosition),
                        id: function (d) { return d.index; }
                    });
                    var clickHandler = function (d, index) {
                        d3.event.preventDefault();
                        var cursorDataPoints = _this.timelineData.cursorDataPoints;
                        var keyEvent = d3.event;
                        if (keyEvent.altKey || keyEvent.shiftKey) {
                            if (_this.timelineData.selectionEndIndex < index) {
                                cursorDataPoints[1].selectionIndex = (d.datePeriod.index + d.datePeriod.fraction);
                                timelineData.selectionEndIndex = index;
                            }
                            else {
                                cursorDataPoints[0].selectionIndex = d.datePeriod.index;
                                timelineData.selectionStartIndex = index;
                            }
                        }
                        else {
                            timelineData.selectionStartIndex = index;
                            timelineData.selectionEndIndex = index;
                            cursorDataPoints[0].selectionIndex = d.datePeriod.index;
                            cursorDataPoints[1].selectionIndex = (d.datePeriod.index + d.datePeriod.fraction);
                        }
                        _this.fillCells(timelineFormat.cellFormat);
                        _this.renderCursors(timelineData, timelineFormat, timelineProperties.cellHeight, timelineProperties.cellsYPosition);
                        _this.renderTimeRangeText(timelineData, timelineFormat.rangeTextFormat);
                        _this.fillColorGranularity(_this.timelineFormat.granularityFormat);
                        _this.setSelection(timelineData);
                    };
                    cellsSelection.on('click', clickHandler).on("touchstart", clickHandler);
                    this.fillCells(timelineFormat.cellFormat);
                    cellsSelection.exit().remove();
                };
                Timeline.prototype.dragstarted = function () {
                    this.timelineData.dragging = true;
                };
                Timeline.prototype.dragged = function (currentCursor) {
                    if (this.timelineData.dragging === true) {
                        var xScale = 1;
                        var container = d3.select(this.timelineSelectors.TimelineVisual.selector);
                        if (container) {
                            var transform = container.style("transform");
                            if (transform !== undefined && transform !== 'none') {
                                var str = transform.split("(")[1];
                                xScale = Number(str.split(", ")[0]);
                            }
                        }
                        var cursorOverElement = this.findCursorOverElement(d3.event.x);
                        if (!cursorOverElement) {
                            return;
                        }
                        var currentlyMouseOverElement = cursorOverElement.datapoint, currentlyMouseOverElementIndex = cursorOverElement.index;
                        if (currentCursor.cursorIndex === 0 && currentlyMouseOverElementIndex <= this.timelineData.selectionEndIndex) {
                            this.timelineData.selectionStartIndex = currentlyMouseOverElementIndex;
                            this.timelineData.cursorDataPoints[0].selectionIndex = currentlyMouseOverElement.datePeriod.index;
                        }
                        if (currentCursor.cursorIndex === 1 && currentlyMouseOverElementIndex >= this.timelineData.selectionStartIndex) {
                            this.timelineData.selectionEndIndex = currentlyMouseOverElementIndex;
                            this.timelineData.cursorDataPoints[1].selectionIndex = (currentlyMouseOverElement.datePeriod.index + currentlyMouseOverElement.datePeriod.fraction);
                        }
                        this.fillCells(this.timelineFormat.cellFormat);
                        this.renderCursors(this.timelineData, this.timelineFormat, this.timelineProperties.cellHeight, this.timelineProperties.cellsYPosition);
                        this.renderTimeRangeText(this.timelineData, this.timelineFormat.rangeTextFormat);
                        this.fillColorGranularity(this.timelineFormat.granularityFormat);
                    }
                };
                /**
                 * Note: Public for testability.
                 */
                Timeline.prototype.findCursorOverElement = function (x) {
                    var timelineDatapoints = this.timelineData.timelineDatapoints || [], length = timelineDatapoints.length, cellWidth = this.timelineProperties.cellWidth;
                    if (timelineDatapoints[0] && timelineDatapoints[1] && x <= timelineDatapoints[1].index * cellWidth) {
                        return {
                            index: 0,
                            datapoint: timelineDatapoints[0]
                        };
                    }
                    else if (timelineDatapoints[length - 1] && x >= timelineDatapoints[length - 1].index * cellWidth) {
                        return {
                            index: length - 1,
                            datapoint: timelineDatapoints[length - 1]
                        };
                    }
                    for (var i = 1; i < length; i++) {
                        var left = timelineDatapoints[i].index * cellWidth, right = timelineDatapoints[i + 1].index * cellWidth;
                        if (x >= left && x <= right) {
                            return {
                                index: i,
                                datapoint: timelineDatapoints[i]
                            };
                        }
                    }
                    return null;
                };
                Timeline.prototype.dragended = function () {
                    this.setSelection(this.timelineData);
                };
                Timeline.prototype.renderCursors = function (timelineData, timelineFormat, cellHeight, cellsYPosition) {
                    var _this = this;
                    var cursorSelection = this.cursorGroupElement.selectAll(this.timelineSelectors.SelectionCursor.selector).data(timelineData.cursorDataPoints);
                    cursorSelection.enter().append('path').classed(this.timelineSelectors.SelectionCursor.class, true);
                    cursorSelection.attr("transform", function (d) { return visuals.SVGUtil.translate(d.selectionIndex * _this.timelineProperties.cellWidth, cellHeight / 2 + cellsYPosition); }).attr({
                        d: d3.svg.arc().innerRadius(0).outerRadius(cellHeight / 2).startAngle(function (d) { return d.cursorIndex * Math.PI + Math.PI; }).endAngle(function (d) { return d.cursorIndex * Math.PI + 2 * Math.PI; })
                    }).call(this.drag);
                    cursorSelection.exit().remove();
                    return cursorSelection;
                };
                Timeline.prototype.renderTimeRangeText = function (timelineData, timeRangeFormat) {
                    var leftMargin = (GranularityNames.length + 2) * this.timelineProperties.elementWidth;
                    var maxWidth = this.svgWidth - leftMargin - this.timelineProperties.leftMargin;
                    if (timeRangeFormat.showProperty && maxWidth > 0) {
                        var timeRangeText = Utils.timeRangeText(timelineData);
                        var labelFormattedTextOptions = {
                            label: timeRangeText,
                            maxWidth: maxWidth,
                            fontSize: timeRangeFormat.sizeProperty
                        };
                        var actualText = visuals.dataLabelUtils.getLabelFormattedText(labelFormattedTextOptions);
                        this.rangeText.classed(this.timelineSelectors.SelectionRangeContainer.class, true);
                        this.rangeText.attr({
                            x: (GranularityNames.length) * (this.timelineProperties.elementWidth + this.timelineProperties.leftMargin),
                            y: 40,
                            fill: timeRangeFormat.colorProperty
                        }).style({
                            'font-size': pt(timeRangeFormat.sizeProperty)
                        }).text(actualText).append('title').text(timeRangeText);
                        ;
                    }
                    else
                        this.rangeText.text("");
                };
                Timeline.prototype.setSelection = function (timelineData) {
                    this.requiresNoUpdate = true;
                    var lower = powerbi.data.SQExprBuilder.dateTime(Utils.getStartSelectionDate(timelineData));
                    var upper = powerbi.data.SQExprBuilder.dateTime(new Date(Utils.getEndSelectionDate(timelineData).getTime() - 1));
                    var filterExpr = powerbi.data.SQExprBuilder.between(timelineData.columnIdentity, lower, upper);
                    var filter = powerbi.data.SemanticFilter.fromSQExpr(filterExpr);
                    var objects = {
                        merge: [
                            {
                                objectName: "general",
                                selector: undefined,
                                properties: {
                                    "filter": filter,
                                }
                            }
                        ]
                    };
                    this.hostServices.persistProperties(objects);
                    this.hostServices.onSelect({ data: [] });
                };
                // This function retruns the values to be displayed in the property pane for each object.
                // Usually it is a bind pass of what the property pane gave you, but sometimes you may want to do
                // validation and return other values/defaults 
                Timeline.prototype.enumerateObjectInstances = function (options) {
                    var enumeration = new visuals.ObjectEnumerationBuilder();
                    switch (options.objectName) {
                        case 'rangeHeader':
                            this.enumerateRangeHeader(enumeration, this.dataView);
                            break;
                        case 'cells':
                            this.enumerateCells(enumeration, this.dataView);
                            break;
                        case 'granularity':
                            this.enumerateGranularity(enumeration, this.dataView);
                            break;
                        case 'labels':
                            this.enumerateLabels(enumeration, this.dataView);
                            break;
                        case 'calendar':
                            this.enumerateCalendar(enumeration, this.dataView);
                            break;
                        case 'weekDay':
                            this.enumerateWeekDay(enumeration, this.dataView);
                            break;
                    }
                    return enumeration.complete();
                };
                Timeline.prototype.enumerateRangeHeader = function (enumeration, dataview) {
                    var objects = dataview && dataview.metadata ? dataview.metadata.objects : undefined;
                    enumeration.pushInstance({
                        objectName: 'rangeHeader',
                        displayName: 'Selection Color',
                        selector: null,
                        properties: {
                            show: powerbi.DataViewObjects.getValue(objects, TimeRangeShowProp, this.defaultTimelineProperties.TimelineDefaultTimeRangeShow),
                            fontColor: powerbi.DataViewObjects.getFillColor(objects, TimeRangeColorProp, this.defaultTimelineProperties.DefaultTimeRangeColor),
                            textSize: powerbi.DataViewObjects.getValue(objects, TimeRangeSizeProp, this.defaultTimelineProperties.TimelineDefaultTextSize)
                        }
                    });
                };
                Timeline.prototype.enumerateCells = function (enumeration, dataview) {
                    var objects = dataview && dataview.metadata ? dataview.metadata.objects : undefined;
                    enumeration.pushInstance({
                        objectName: 'cells',
                        selector: null,
                        properties: {
                            fillSelected: powerbi.DataViewObjects.getFillColor(objects, SelectedCellColorProp, this.defaultTimelineProperties.TimelineDefaultCellColor),
                            fillUnselected: powerbi.DataViewObjects.getFillColor(objects, UnselectedCellColorProp, this.defaultTimelineProperties.TimelineDefaultCellColorOut)
                        }
                    });
                };
                Timeline.prototype.enumerateGranularity = function (enumeration, dataview) {
                    var objects = dataview && dataview.metadata ? dataview.metadata.objects : undefined;
                    enumeration.pushInstance({
                        objectName: 'granularity',
                        selector: null,
                        properties: {
                            scaleColor: powerbi.DataViewObjects.getFillColor(objects, ScaleColorProp, this.defaultTimelineProperties.DefaultScaleColor),
                            sliderColor: powerbi.DataViewObjects.getFillColor(objects, SliderColorProp, this.defaultTimelineProperties.DefaultSliderColor),
                        }
                    });
                };
                Timeline.prototype.enumerateLabels = function (enumeration, dataview) {
                    var objects = dataview && dataview.metadata ? dataview.metadata.objects : undefined;
                    enumeration.pushInstance({
                        objectName: 'labels',
                        selector: null,
                        properties: {
                            show: powerbi.DataViewObjects.getValue(objects, LabelsShowProp, this.defaultTimelineProperties.DefaultLabelsShow),
                            fontColor: powerbi.DataViewObjects.getFillColor(objects, LabelsColorProp, this.defaultTimelineProperties.DefaultLabelColor),
                            textSize: powerbi.DataViewObjects.getValue(objects, LabelsSizeProp, this.defaultTimelineProperties.TimelineDefaultTextSize)
                        }
                    });
                };
                Timeline.prototype.enumerateCalendar = function (enumeration, dataview) {
                    var objects = dataview && dataview.metadata ? dataview.metadata.objects : undefined;
                    enumeration.pushInstance({
                        objectName: 'calendar',
                        selector: null,
                        properties: {
                            month: Math.max(1, Math.min(12, powerbi.DataViewObjects.getValue(objects, CalendarMonthProp, 1))),
                            day: Math.max(1, Math.min(31, powerbi.DataViewObjects.getValue(objects, CalendarDayProp, 1))),
                        }
                    });
                };
                Timeline.prototype.enumerateWeekDay = function (enumeration, dataview) {
                    var objects = dataview && dataview.metadata ? dataview.metadata.objects : undefined;
                    enumeration.pushInstance({
                        objectName: 'weekDay',
                        selector: null,
                        properties: {
                            day: Math.max(0, Math.min(6, powerbi.DataViewObjects.getValue(objects, WeekDayProp, 0)))
                        }
                    });
                };
                Timeline.capabilities = {
                    dataRoles: [{
                        name: 'Time',
                        kind: powerbi.VisualDataRoleKind.Grouping,
                        displayName: 'Time'
                    }],
                    dataViewMappings: [{
                        conditions: [
                            { 'Time': { max: 1 } }
                        ],
                        categorical: {
                            categories: {
                                for: { in: 'Time' },
                                dataReductionAlgorithm: { sample: {} }
                            },
                            values: {
                                select: [{
                                    bind: { to: 'Time' }
                                }]
                            },
                        }
                    }],
                    objects: {
                        general: {
                            displayName: 'General',
                            properties: {
                                formatString: {
                                    type: {
                                        formatting: {
                                            formatString: true
                                        }
                                    },
                                },
                                selected: {
                                    type: { bool: true }
                                },
                                filter: {
                                    type: { filter: {} },
                                    rule: {
                                        output: {
                                            property: 'selected',
                                            selector: ['Time'],
                                        }
                                    }
                                },
                            },
                        },
                        calendar: {
                            displayName: 'Fiscal Year Start',
                            properties: {
                                month: {
                                    displayName: 'Month',
                                    type: { enumeration: Timeline1447991079100.Months }
                                },
                                day: {
                                    displayName: 'Day',
                                    type: { numeric: true }
                                }
                            }
                        },
                        weekDay: {
                            displayName: 'First Day of Week',
                            properties: {
                                day: {
                                    displayName: 'Day',
                                    type: { enumeration: Timeline1447991079100.WeekDays }
                                }
                            }
                        },
                        rangeHeader: {
                            displayName: 'Range Header',
                            properties: {
                                show: {
                                    displayName: 'Show',
                                    type: { bool: true }
                                },
                                fontColor: {
                                    displayName: 'Font color',
                                    type: { fill: { solid: { color: true } } }
                                },
                                textSize: {
                                    displayName: 'Text Size',
                                    type: { numeric: true }
                                }
                            }
                        },
                        cells: {
                            displayName: 'Cells',
                            properties: {
                                fillSelected: {
                                    displayName: 'Selected cell color',
                                    type: { fill: { solid: { color: true } } }
                                },
                                fillUnselected: {
                                    displayName: 'Unselected cell color',
                                    type: { fill: { solid: { color: { nullable: true } } } }
                                }
                            }
                        },
                        granularity: {
                            displayName: 'Granularity',
                            properties: {
                                scaleColor: {
                                    displayName: 'Scale color',
                                    type: { fill: { solid: { color: true } } }
                                },
                                sliderColor: {
                                    displayName: 'Slider color',
                                    type: { fill: { solid: { color: true } } }
                                }
                            }
                        },
                        labels: {
                            displayName: 'Labels',
                            properties: {
                                show: {
                                    displayName: 'Show',
                                    type: { bool: true }
                                },
                                fontColor: {
                                    displayName: 'Font color',
                                    type: { fill: { solid: { color: true } } }
                                },
                                textSize: {
                                    displayName: 'Text Size',
                                    type: { numeric: true }
                                }
                            }
                        }
                    }
                };
                return Timeline;
            })();
            Timeline1447991079100.Timeline = Timeline;
        })(Timeline1447991079100 = visuals.Timeline1447991079100 || (visuals.Timeline1447991079100 = {}));
    })(visuals = powerbi.visuals || (powerbi.visuals = {}));
})(powerbi || (powerbi = {}));
var powerbi;
(function (powerbi) {
    var visuals;
    (function (visuals) {
        var plugins;
        (function (plugins) {
            plugins.Timeline1447991079100 = {
                name: 'Timeline1447991079100',
                class: 'Timeline1447991079100',
                capabilities: powerbi.visuals.Timeline1447991079100.Timeline.capabilities,
                custom: true,
                create: function (options) { return new powerbi.visuals.Timeline1447991079100.Timeline(options); },
                apiVersion: null
            };
        })(plugins = visuals.plugins || (visuals.plugins = {}));
    })(visuals = powerbi.visuals || (powerbi.visuals = {}));
})(powerbi || (powerbi = {}));
